function out = ftransp(in, dx, dy, roundpos)
%% Forward registration

% Copyright 2018 Laurent Hoeltgen <contact@laurenthoeltgen.name>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
% Street, Fifth Floor, Boston, MA 02110-1301, USA.

% Last revision on: 03.09.2018 10:34

out = zeros(size(in));
[nr, nc] = size(in);

[ir, jc, va] = find(in);

for kk = 1:numel(ir)
    
    if roundpos
        sx = round(dx(ir(kk),jc(kk)));
        sy = round(dy(ir(kk),jc(kk)));
        ex = 0;
        ey = 0;
    else
        sx = floor(dx(ir(kk),jc(kk)));
        sy = floor(dy(ir(kk),jc(kk)));
        ex = dx(ir(kk),jc(kk)) - sx;
        ey = dy(ir(kk),jc(kk)) - sy;
    end
    
    tr = ir(kk) + sy;
    tc = jc(kk) + sx;
    
    if (tr >= 1) && (tr < nr) && (tc >= 1) && (tc < nc)
        out(tr + 0, tc + 0) = out(tr + 0, tc + 0) + (1-ey)*(1-ex)*va(kk);
        out(tr + 1, tc + 0) = out(tr + 1, tc + 0) + (0+ey)*(1-ex)*va(kk);
        out(tr + 0, tc + 1) = out(tr + 0, tc + 1) + (1-ey)*(0+ex)*va(kk);
        out(tr + 1, tc + 1) = out(tr + 1, tc + 1) + (0+ey)*(0+ex)*va(kk);
    elseif (tr == nr) && (tc >= 1) && (tc <  nc)
        out(tr + 0, tc + 0) = out(tr + 0, tc + 0) + (1-ex)*va(kk);
        out(tr + 0, tc + 1) = out(tr + 0, tc + 1) + (0+ex)*va(kk);
    elseif (tr >= 1) && (tr <  nr) && (tc == nc)
        out(tr + 0, tc + 0) = out(tr + 0, tc + 0) + (1-ey)*va(kk);
        out(tr + 1, tc + 0) = out(tr + 1, tc + 0) + (0+ey)*va(kk);
    elseif (tr == nr) && (tc == nc)
        out(tr + 0, tc + 0) = out(tr + 0, tc + 0) + va(kk);
    end
end

end
