function d = moveMaskInterp(c, flow)

% Copyright 2019 Georg Radow <radow@b-tu.de>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
% Street, Fifth Floor, Boston, MA 02110-1301, USA.

% Last revision on: 27.08.2019 10:09

[nr, nc, ~, nd] = size(flow);

d = zeros(nr, nc, nd + 1);

% initialise lists of values and coordinates
val = zeros(nr*nc,1);
x = zeros(nr*nc,1);
y = zeros(nr*nc,1);

% number of entries
nnz = 0;
% fill values and coordinates
for jj=1:nc
    for ii=1:nr
        if c(ii,jj)>0
            nnz = nnz+1;
            val(nnz) = c(ii,jj);
            x(nnz) = jj;
            y(nnz) = ii;
            
        end
    end
end

% crop vectors
val = val(1:nnz,1);
x = x(1:nnz,1);
y = y(1:nnz,1);
% booleans to indicate if a location is currently active
active = true(nnz,1);



d(:,:,1) = c;


for ii = 1:nd
    
    % get location for bilinear interpolation
    xInt = floor(x);
    yInt = floor(y);
    xDev = x - xInt;
    yDev = y - yInt;
    
    % get padded flow fields (for interpolation at the boundary)
    u = [flow(:,:,1,ii), zeros(nr,1); zeros(1,nc+1)];
    v = [flow(:,:,2,ii), zeros(nr,1); zeros(1,nc+1)];
    
    
    % bilinear interpolation
    for kk=1:length(x)
        weights = [(1-yDev(kk));yDev(kk)] * [(1-xDev(kk)),xDev(kk)]; 
        x(kk) = x(kk)+sum(sum(weights .* ...
            u(yInt(kk):(yInt(kk)+1),xInt(kk):(xInt(kk)+1))));
        y(kk) = y(kk)+sum(sum(weights .* ...
            v(yInt(kk):(yInt(kk)+1),xInt(kk):(xInt(kk)+1))));
        
    end
    
    % shift values that are within the image but not within the grid of
    % pixel locations, motivated by the behaviour of ftransp( ., ., true)
    x((x<1)&(x>=0.5)) = 1;
    x((x>nc)&(x<(nc+0.5))) = nc;
    y((y<1)&(y>=0.5)) = 1;
    y((y>nr)&(y<(nr+0.5))) = nr;
    
    % check if locations are still active
    active(x<1) = false;
    active(x>nc) = false;
    active(y<1) = false;
    active(y>nr) = false;
    % crop working variables
    val = val(active);
    x = x(active);
    y = y(active);
    active = active(active);
    
    % write new mask at rounded positions
    tmp = zeros(nr,nc);
    tmp(nr*(round(x)-1)+round(y)) = val;
    % crop values (if two mask locations have been merged)
    tmp(tmp>1) = 1;
    
    d(:,:,ii+1) = tmp;
end

end