
clear;

%% optimizer settings
% quadratic relaxation parameter
theta = 0.25; % theta in [0.1, 0.5]

% TV norm with quadratic behavior between 0 and epsilon
epsilon = 0.1;

% use edge weighted TV
use_edges = 1; % 0 = no, 1 = yes

% number of TV-L1 iterations per warp
maxits = 200;

% warping parameters
max_pyramid_levels = 8; % max number of pyramid levels
pyramid_factor = 0.5; % in [0.5, 0.95]
warps = 10; % number of warps per level

% preprocessing
use_structure_texture = 1;
structure_texture_sigma = 1.0;
structure_texture_factor = 0.8;

%% line search
load('../hs/yosemite-complete.mat');

optLambdaTp = zeros(1,(size(ff,3)-1));
aaeTp = zeros(1,(size(ff,3)-1));
u3 = zeros(size(u1));
v3 = zeros(size(u1));
err=zeros(1,21);


for iImg=1:(size(ff,3)-1)
    vcx = fu(:,:,iImg);
    vcy = fv(:,:,iImg);
    
    start = 10;
    
    for digit=1:-1:-2
        alpha = linspace(start-10^digit,start+10^digit,21);
        
        for ii=1:length(alpha)
            if alpha(ii)>0
                [flow] = coarse_to_fine(ff(:,:,iImg), ff(:,:,iImg), ff(:,:,iImg+1), alpha(ii), theta, epsilon, use_edges, maxits, ...
                    max_pyramid_levels, pyramid_factor, warps, ...
                    use_structure_texture, structure_texture_sigma, structure_texture_factor);
                vex = flow(:,:,1);
                vey = flow(:,:,2);
                tmp = arrayfun(@(x) real(acos(x))*180/pi, ...
                    (vcx.*vex + vcy.*vey + 1)./(sqrt(vcx.^2 + vcy.^2 + 1) ...
                    .* sqrt(vex.^2 +vey.^2 + 1)));
                err(ii) = sum(tmp(:))/numel(vcx);
            else
                err(ii) = inf;
            end
        end
        
        [aae,iOpt] = min(err);
        start = alpha(iOpt);
    end
    
    aaeTp(iImg) = aae;
    optLambdaTp(iImg) = start;
    [flow] = coarse_to_fine(ff(:,:,iImg), ff(:,:,iImg), ff(:,:,iImg+1), start, theta, epsilon, use_edges, maxits, ...
        max_pyramid_levels, pyramid_factor, warps, ...
        use_structure_texture, structure_texture_sigma, structure_texture_factor);
    
    u3(:,:,iImg) = flow(:,:,1);
    v3(:,:,iImg) = flow(:,:,2);
    save('../hs/yosemite-complete.mat','ff','fu','fv','u1','u2','u3','u4', ...
        'v1','v2','v3','v4', ...
        'optLambdaDs','optLambdaMa','optLambdaTp','optLambdaL1', ...
        'aaeDs','aaeMa','aaeTp','aaeL1');
    
end